package dev.kske.chess.board;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import dev.kske.chess.board.Piece.Color;
import dev.kske.chess.board.Piece.Type;

/**
 * Project: <strong>Chess</strong><br>
 * File: <strong>Board.java</strong><br>
 * Created: <strong>01.07.2019</strong><br>
 * Author: <strong>Kai S. K. Engelbart</strong>
 */
public class Board {

	private Piece[][]				boardArr;
	private Map<Color, Position>	kingPos;

	public Board() {
		boardArr			= new Piece[8][8];
		kingPos				= new HashMap<>();
		initializeDefaultPositions();
	}

	/**
	 * Moves a piece across the board if the move is legal.
	 * 
	 * @param move The move to execute
	 * @return {@code true}, if the attempted move was legal and thus executed
	 */
	public boolean attemptMove(Move move) {
		Piece piece = getPos(move);
		if (piece == null || !piece.isValidMove(move)) return false;
		else {
			/*
			 * Move piece
			 * Save destination piece for possible canceling of the move
			 */
			Piece capturePiece = move(move);

			// Revert move if it caused a check for its team
			if (checkCheck(piece.getColor())) {
				revert(move, capturePiece);
				return false;
			}

			return true;
		}
	}

	/**
	 * Moves a piece across the board without checking if the move is legal.
	 * 
	 * @param move The move to execute
	 * @return The captures piece, or null if the move's destination was empty
	 */
	public Piece move(Move move) {
		Piece	piece			= getPos(move);
		Piece	capturePiece	= getDest(move);
		setDest(move, piece);
		setPos(move, null);

		// Update the king's position if the moved piece is the king
		if (piece.getType() == Type.KING) kingPos.put(piece.getColor(), move.dest);

		return capturePiece;
	}

	/**
	 * Reverts a move.
	 * 
	 * @param move          The move to revert
	 * @param capturedPiece The piece that has been captured when the move has been
	 *                      applied
	 */
	public void revert(Move move, Piece capturedPiece) {
		setPos(move, getDest(move));
		setDest(move, capturedPiece);

		// Update the king's position if the moved piece is the king
		if (getPos(move).getType() == Type.KING) kingPos.put(getPos(move).getColor(), move.pos);
	}

	/**
	 * Generated every legal move for one color
	 * 
	 * @param color The color to generate the moves for
	 * @return A list of all legal moves
	 */
	public List<Move> getMoves(Color color) {
		List<Move> moves = new ArrayList<>();
		for (int i = 0; i < 8; i++)
			for (int j = 0; j < 8; j++)
				if (boardArr[i][j] != null && boardArr[i][j].getColor() == color)
					moves.addAll(boardArr[i][j].getMoves(new Position(i, j)));
		return moves;
	}

	public List<Move> getMoves(Position pos) {
		return get(pos).getMoves(pos);
	}

	/**
	 * Checks, if the king is in check.
	 * 
	 * @param color The color of the king to check
	 * @return {@code true}, if the king is in check
	 */
	public boolean checkCheck(Color color) {
		for (int i = 0; i < 8; i++)
			for (int j = 0; j < 8; j++) {
				Position pos = new Position(i, j);
				if (get(pos) != null && get(pos).getColor() != color
						&& get(pos).isValidMove(new Move(pos, kingPos.get(color))))
					return true;
			}
		return false;
	}

	/**
	 * Checks, if the king is in checkmate.
	 * This requires the king to already be in check!
	 * 
	 * @param color The color of the king to check
	 * @return {@code true}, if the king is in checkmate
	 */
	public boolean checkCheckmate(Color color) {
		// Return false immediately if the king can move
		if (!getMoves(kingPos.get(color)).isEmpty()) return false;
		else {
			for (Move move : getMoves(color)) {
				Piece	capturePiece	= move(move);
				boolean	check			= checkCheck(color);
				revert(move, capturePiece);
				if (!check) return false;
			}
			return true;
		}
	}

	public GameState getGameEventType(Color color) {
		return checkCheck(color)
				? checkCheckmate(color) ? GameState.CHECKMATE : GameState.CHECK
				: getMoves(color).isEmpty() ? GameState.STALEMATE : GameState.NORMAL;
	}

	/**
	 * Evaluated the board.
	 * 
	 * @param color The color to evaluate for
	 * @return An positive number representing how good the position is
	 */
	public int evaluate(Color color) {
		int score = 0;
		for (int i = 0; i < 8; i++)
			for (int j = 0; j < 8; j++)
				if (boardArr[i][j] != null && boardArr[i][j].getColor() == color) switch (boardArr[i][j].getType()) {
					case QUEEN:
						score += 8;
						break;
					case ROOK:
						score += 5;
						break;
					case KNIGHT:
						score += 3;
						break;
					case BISHOP:
						score += 3;
						break;
					case PAWN:
						score += 1;
						break;
				}
		return score;
	}

	/**
	 * Initialized the board array with the default chess pieces and positions.
	 */
	public void initializeDefaultPositions() {
		// Initialize pawns
		for (int i = 0; i < 8; i++) {
			boardArr[i][1]	= new Pawn(Color.BLACK, this);
			boardArr[i][6]	= new Pawn(Color.WHITE, this);
		}

		// Initialize kings
		boardArr[4][0]	= new King(Color.BLACK, this);
		boardArr[4][7]	= new King(Color.WHITE, this);

		// Initialize king position objects
		kingPos.put(Color.BLACK, new Position(4, 0));
		kingPos.put(Color.WHITE, new Position(4, 7));

		// Initialize queens
		boardArr[3][0]	= new Queen(Color.BLACK, this);
		boardArr[3][7]	= new Queen(Color.WHITE, this);

		// Initialize rooks
		boardArr[0][0]	= new Rook(Color.BLACK, this);
		boardArr[0][7]	= new Rook(Color.WHITE, this);
		boardArr[7][0]	= new Rook(Color.BLACK, this);
		boardArr[7][7]	= new Rook(Color.WHITE, this);

		// Initialize knights
		boardArr[1][0]	= new Knight(Color.BLACK, this);
		boardArr[1][7]	= new Knight(Color.WHITE, this);
		boardArr[6][0]	= new Knight(Color.BLACK, this);
		boardArr[6][7]	= new Knight(Color.WHITE, this);

		// Initialize bishops
		boardArr[2][0]	= new Bishop(Color.BLACK, this);
		boardArr[2][7]	= new Bishop(Color.WHITE, this);
		boardArr[5][0]	= new Bishop(Color.BLACK, this);
		boardArr[5][7]	= new Bishop(Color.WHITE, this);

		// Clear all other tiles
		for (int i = 0; i < 8; i++)
			for (int j = 2; j < 6; j++)
				boardArr[i][j] = null;
	}

	public Piece get(Position pos) {
		return boardArr[pos.x][pos.y];
	}

	public void set(Position pos, Piece piece) {
		boardArr[pos.x][pos.y] = piece;
	}

	public Piece getPos(Move move) {
		return get(move.pos);
	}

	public Piece getDest(Move move) {
		return get(move.dest);
	}

	public void setPos(Move move, Piece piece) {
		set(move.pos, piece);
	}

	public void setDest(Move move, Piece piece) {
		set(move.dest, piece);
	}

	/**
	 * @return The board array
	 */
	public Piece[][] getBoardArr() { return boardArr; }
}
